;################################################################################
;#										#
;# chipbasic - single chip basic computer with ATMega 16			#
;# full screen editor								#
;# copyright (c) 2006 Joerg Wolfram (joerg@jcwolfram.de)			#
;#										#
;#										#
;# This program is free software; you can redistribute it and/or		#
;# modify it under the terms of the GNU General Public License			#
;# as published by the Free Software Foundation; either version 2		#
;# of the License, or (at your option) any later version.			#
;#										#
;# This program is distributed in the hope that it will be useful,		#
;# but WITHOUT ANY WARRANTY; without even the implied warranty of		#
;# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU		#
;# General Public License for more details.					#
;#										#
;# You should have received a copy of the GNU General Public			#
;# License along with this library; if not, write to the			#
;# Free Software Foundation, Inc., 59 Temple Place - Suite 330,			#
;# Boston, MA 02111-1307, USA.							#
;#										#
;################################################################################

.set	editor_linestart = 2
.set	editor_linedigits = 2
.set	bas_linelen = 27
.set	editor_unchanged = 19
.set	editor_changed = 21

;------------------------------------------------------------------------------
;main screen mask
;------------------------------------------------------------------------------
editor_entry:	libmio_clrscr			;clear screen
		libmio_mode12			;set screen in editor mode
		clr	ctrl			;set mode
		mov	r13,byte0		;first viewed line

		libmio_thistext
		.db 0,0,editor_unchanged," PROGRAM: ",0
		ldi	ZL,LOW(libmio_vram+11)	;start of program name
		ldi	ZH,HIGH(libmio_vram+11)
		rcall	ee_readname
		rcall	editor_cstat		;clear status line
		ldi	YL,0
		ldi	YH,2
		ldi	ZL,61
		ldi	ZH,2
		libmio_fbox
		call	lce_pload		;load program fro EEPROM	
		cpi	ereg,0
		breq	editor_entry_p
editor_lnum_2:	ldi	XL,21			;yellow
		ldi	ZL,LOW(errtab1*2)
		ldi	ZH,HIGH(errtab1*2)
editor_lnum_3:	cpi	ereg,1
		breq	editor_entry_o
		ldi	XL,17			;red
		dec	ereg
editor_lnum_4:	lpm	tempreg1,Z+
		cpi	tempreg1,0
		brne	editor_lnum_4
		rjmp	editor_lnum_3	

editor_entry_o:	rcall	editor_fstat		;fill status line
		ldi	ctrl,0x02
		ldi	XL,2
		ldi	XH,2
		libmio_gotoxy
		libmio_romtext			;errortext
		libmio_thistext
		.db	255," IN ",0
		clr	XH
		mov	XL,r12			;line no	
		inc	XL
    		libmio_outdez
		ldi	tempreg1,':'
		libmio_outchar
		mov	XL,r13			;statement	
		inc	XL
    		libmio_outdez
editor_entry_p:	ret				;thats all
	
editor_cset:	ldi	XL,editor_changed		;change signal char
		mov	tempreg6,XL
		ldi	tempreg4,editor_unchanged
		ret
	
;------------------------------------------------------------------------------
;main editor loop
;------------------------------------------------------------------------------
editor_main:	rcall	editor_cset
		rcall	editor_entry
editor_main_00:	ldi	XL,3
		ldi	XH,3
editor_main_0:	mov	libmio_cur_x,XL
		mov	libmio_cur_y,XH
editor_main_1:	libmio_cursor

;------------------------------------------------------------------------------
;pos1 key
;------------------------------------------------------------------------------
		cpi	tempreg1,0xe0		;pos1
		brne	editor_main_2
		ldi	XL,3
		rjmp	editor_main_0

;------------------------------------------------------------------------------
;end key
;------------------------------------------------------------------------------
editor_main_2:	cpi	tempreg1,0xe1		;end
		brne	editor_main_3
		ldi	XL,libmio_cols-1
		rjmp	editor_main_0

;------------------------------------------------------------------------------
;cursor left key
;------------------------------------------------------------------------------
editor_main_3:	cpi	tempreg1,0xe2		;left
		brne	editor_main_4
		dec	XL
		rjmp	editor_checkb

;------------------------------------------------------------------------------
;cursor right key
;------------------------------------------------------------------------------
editor_main_4:	cpi	tempreg1,0xe3		;right
		brne	editor_main_5
		inc	XL	
		rjmp	editor_checkb

;------------------------------------------------------------------------------
;cursor up key
;------------------------------------------------------------------------------
editor_main_5:	cpi	tempreg1,0xe4		;up
		brne	editor_main_6
		dec	XH
		rjmp	editor_checkb

;------------------------------------------------------------------------------
;cursor down key
;------------------------------------------------------------------------------
editor_main_6:	cpi	tempreg1,0xe5		;down
		brne	editor_main_7
		inc	XH
		rjmp	editor_checkb

;------------------------------------------------------------------------------
;enter key
;------------------------------------------------------------------------------
editor_main_7:	cpi	tempreg1,0xea		;enter
		brne	editor_main_70
		inc	XH
		ldi	XL,3
		rjmp	editor_checkb

;------------------------------------------------------------------------------
;limit coordinates to editable area
;------------------------------------------------------------------------------
editor_checkb:	cpi	XL,3
		brcc	editor_chb1
		ldi	XL,3
editor_chb1:	cpi	XL,libmio_cols
		brcs	editor_chb2
		ldi	XL,libmio_cols-1			
editor_chb2:	cpi	XH,3
		brcc	editor_chb3
		ldi	XH,3
editor_chb3:	cpi	XH,3+bas_lines
		brcs	editor_chb4
		ldi	XH,3+bas_lines-1
editor_chb4:	mov	r16,XH			;y-coord
		subi	r16,3			;correct	
		cp	r16,r13			;check if line is visible
		brcc	editor_chb5
		mov	r13,r16
editor_chb5:	mov	r16,r13			;y-coord
		subi	r16,0xee		;+18
		cp	r16,XH			;check if line is visible
		brcc	editor_chb6
		mov	r16,XH
		subi	r16,0x12
		mov	r13,r16
editor_chb6:	rjmp	editor_main_0			

;------------------------------------------------------------------------------
;alt+delete key
;------------------------------------------------------------------------------
editor_main_70:	lds	tempreg2,libmio_kbdstate
		sbrs	tempreg2,4
		rjmp	editor_main_8
		mov	ctrl,XH
		subi	ctrl,3
		cpi	tempreg1,0xe9		;delete
		brne	editor_main_75
		push	XL
		push	XH
		call	lce_del
		pop	XH
		pop	XL		
		rjmp	editor_change

;------------------------------------------------------------------------------
;alt+insert key (insert line)
;------------------------------------------------------------------------------
editor_main_75:	cpi	tempreg1,0xe8		;insert
		brne	editor_main_8
		push	XL
		push	XH
		call	lce_ins
		pop	XH
		pop	XL		
		rjmp	editor_change
		
;------------------------------------------------------------------------------
;delete key
;------------------------------------------------------------------------------
editor_main_8:	cpi	tempreg1,0xe9		;delete
		brne	editor_main_9
		rcall	editor_dchar
		rjmp	editor_change

;------------------------------------------------------------------------------
;backspace key
;------------------------------------------------------------------------------
editor_main_9:	cpi	tempreg1,0xec		;backspace
		brne	editor_main_a
		mov	XL,libmio_cur_x		;actual cursor
		cpi	XL,editor_linedigits+1	;at the beginning?
		brne	editor_main_91		;no
		rjmp	editor_main_1		;loop
editor_main_91:	dec	XL
		mov	libmio_cur_x,XL
		rcall	editor_dchar
		rjmp	editor_change		;loop

;------------------------------------------------------------------------------
;F2 save
;------------------------------------------------------------------------------
editor_main_a:	cpi	tempreg1,0xf2
		brne	editor_main_c
		rcall	editor_uc
		breq	editor_main_b1
		ldi	XL,21
		rcall	editor_fstat		;clear status line
		libmio_thistext
		.db 2,2,"SAVE PROGRAM Y/N ?",0
		rcall	editor_yn		
		brtc	editor_main_b
		call	lce_psave
		rjmp	editor_main
editor_main_b:	rcall	editor_cstat
		clr	ereg
editor_main_b1:	rjmp	editor_main_00			

;------------------------------------------------------------------------------
;F3 Load
;------------------------------------------------------------------------------
editor_main_c:	cpi	tempreg1,0xf3
		brne	editor_main_d
		rcall	editor_uc
		breq	editor_main_c1
		ldi	XL,21
		rcall	editor_fstat		;clear status line
		libmio_thistext
		.db 2,2,"LOST CHANGES Y/N ?",0
		rcall	editor_yn		
		brtc	editor_main_b
editor_main_c1:	rjmp	editor_main

;------------------------------------------------------------------------------
;F8 new program
;------------------------------------------------------------------------------
editor_main_d:	cpi	tempreg1,0xf8
		brne	editor_mainr
		ldi	XL,21
		rcall	editor_fstat		;clear status line
		libmio_thistext
		.db 2,2,"CLEAR PROGRAM Y/N ?",0
		rcall	editor_yn		
		brtc	editor_main_b
		call	lce_pnew
		rcall	editor_cset
		sts	libmio_vram,tempreg6	;set change symbol
editor_main_e:	rjmp	editor_main_b

;------------------------------------------------------------------------------
;F9 edit programm name
;------------------------------------------------------------------------------
editor_mainr:	cpi	tempreg1,0xf9
		brne	editor_exit
		mov	tempreg7,libmio_cur_x
		mov	tempreg8,libmio_cur_y
		rcall	editor_ren
		ldi	ZL,LOW(libmio_vram+11)	;start of program name
		ldi	ZH,HIGH(libmio_vram+11)
		rcall	ee_writename
		mov	libmio_cur_x,tempreg7
		mov	libmio_cur_y,tempreg8
		rjmp	editor_main_1

;------------------------------------------------------------------------------
;ESC
;------------------------------------------------------------------------------
editor_exit:	cpi	tempreg1,0xed
		brne	editor_main_k
		rcall	editor_uc
		brne	editor_exit_1
		libmio_mode10
		ret
editor_exit_1:	ldi	XL,21
		rcall	editor_fstat
		libmio_thistext
		.db 2,2,"QUIT W/O SAVE Y/N ?",0
		rcall	editor_yn		
		cpi	tempreg1,'Y'
		brne	editor_main_e
editor_exit_2:	lds	XL,libmio_keycode
		cpi	XL,0
		brne	editor_exit_2		
		libmio_mode10
		ret

;------------------------------------------------------------------------------
;F10 run
;------------------------------------------------------------------------------
editor_main_k:	cpi	tempreg1,0xfa		;F10
		brne	editor_main_f		;no
		rcall	editor_uc
		brne	editor_main_l
		libmio_clrscr			;clear screen
		libmio_mode10
		ret				;go back and run
editor_main_l:	ldi	XL,18			;magenta
		rcall	editor_fstat
		libmio_thistext
		.db 2,2,"SAVE CHANGES BEFORE RUN!",0
		libmio_waitkey		
		rjmp	editor_main_b

editor_main_f:
;------------------------------------------------------------------------------
;a visible  char
;------------------------------------------------------------------------------
editor_main_20:	lds	XL,libmio_kbdstate
		sbrc	XL,2			;left ctrl
		rjmp	editor_main_ig		;ignore
		sbrc	XL,3			;right ctrl
		rjmp	editor_main_ig		;ignore
		cpi	tempreg1,0x1c		;degree
		brcs	editor_main_ig		;ignore less
		cpi	tempreg1,0x60		;last visible+1
		brcc	editor_main_ig		;igore greater or equal
		mov	XL,libmio_cur_x		;cursor position x
		cpi	XL,libmio_cols-1	;at the end?
		breq	editor_main_28
		mov	tempreg2,XL		;save x-pos
		ldi	XL,libmio_cols-1
		mov	libmio_cur_x,XL
		libmio_calc
editor_main_21:	ld	XH,-Y			;char on cursor pos		
		std	Y+1,XH			;write to next
		dec	XL
		cp	XL,tempreg2
		brne	editor_main_21
		mov	libmio_cur_x,XL
		libmio_calc
    		st	Y,tempreg1
		inc	XL
		mov	libmio_cur_x,XL
		libmio_calc
		sts	libmio_vram,tempreg6		;change symbol
		rjmp	editor_main_1		
				
editor_main_28:	libmio_calc			;calculate vram address
    		st	Y,tempreg1
editor_change:	sts	libmio_vram,tempreg6	;set change symbol
		rjmp	editor_main_1		;loop
		
editor_main_ig:	rjmp	editor_main_1		

;------------------------------------------------------------------------------
;del char on cursor pos
;------------------------------------------------------------------------------
editor_dchar:	ldi	YL,LOW(libmio_vram)
		ldi	YH,HIGH(libmio_vram)
		mov	XL,libmio_cur_y		;cursor position y
		ldi	XH,libmio_cols
		mul	XL,XH
		add	YL,r0
		adc	YH,r1
		mov	XL,libmio_cur_x		;cursor position x
		add	YL,XL
		adc	YH,byte0
editor_dchar_1:	cpi	XL,libmio_cols-1	;at the end?
		breq	editor_dchar_2
		ldd	XH,Y+1
		st	Y+,XH
		inc	XL
		rjmp	editor_dchar_1	
		
editor_dchar_2:	ldi	XH,0x20			;space
		st	Y,XH
		ret


;------------------------------------------------------------------------------
;check keys for Y/N 
;------------------------------------------------------------------------------
editor_yn:	libmio_waitkey
		cpi	tempreg1,'Y'
		breq	editor_yn1
		cpi	tempreg1,0xea		;ENTER
		breq	editor_yn1
		cpi	tempreg1,'N'
		breq	editor_yn2
		cpi	tempreg1,0xed		;ESC
		breq	editor_yn2
		rjmp	editor_yn
		
editor_yn1:	set				;set T-flag
		ret		
editor_yn2:	clt				;res T-flag
		ret		
		
;------------------------------------------------------------------------------
;save program to EEPROM
;------------------------------------------------------------------------------
editor_save:	ldi	XL,0x00			;first line
editor_save1:	push	XL
		rcall	ee_writeline
		pop	XL
		inc	XL
		cpi	XL,bas_lines
		brne	editor_save1
		ldi	tempreg4,editor_unchanged
		sts	libmio_vram,tempreg4	;delete change symbol
		ret	

;------------------------------------------------------------------------------
;check for unchanged-symbol
;------------------------------------------------------------------------------
editor_uc:	lds	XL,libmio_vram
		cpi	XL,editor_unchanged	;no change
		ret
		
		
;------------------------------------------------------------------------------
;clear status line 
;------------------------------------------------------------------------------
editor_cstat:	ldi	XL,19
		rcall	editor_fstat
		libmio_thistext
		.db 2,2,19,"OK",0
		ret

;------------------------------------------------------------------------------
;fill status line with XL 
;------------------------------------------------------------------------------
editor_fstat:	push	ZL
		push	ZH
		ldi	ZL,LOW(libmio_vram+libmio_cols*2)	
		ldi	ZH,HIGH(libmio_vram+libmio_cols*2)	
		ldi	XH,libmio_cols
editor_fstat1:	st	Z+,XL
		dec	XH
		brne	editor_fstat1
		pop	ZH
		pop	ZL		
		ret

;-----------------------------------------------------------------------
; edit name
;-----------------------------------------------------------------------
editor_ren:	ldi	XH,0
		ldi	XL,11
		libmio_gotoxy
edit_ren_01:	libmio_cursor			;view cursor
		cpi	tempreg1,0xea		;enter
		brne	edit_ren_02
		ret
edit_ren_02:	cpi	tempreg1,0xe2		;cursor left
		brne	edit_ren_03
		mov	XL,libmio_cur_x
		cpi	XL,11
		breq	edit_ren_01
		dec	XL
		mov	libmio_cur_x,XL
		rjmp	edit_ren_01
edit_ren_03:	cpi	tempreg1,0xe3		;cursor right
		brne	edit_ren_04
		mov	XL,libmio_cur_x
		cpi	XL,20
		breq	edit_ren_01
		inc	XL
		mov	libmio_cur_x,XL
		rjmp	edit_ren_01
edit_ren_04:	cpi	tempreg1,0x20
		brcs	edit_ren_01			
		cpi	tempreg1,0x60
		brcc	edit_ren_01
		st	Y,tempreg1
		mov	XL,libmio_cur_x
		cpi	XL,20
		breq	edit_ren_05
		inc	XL
		mov	libmio_cur_x,XL
edit_ren_05:	rjmp	edit_ren_01

;------------------------------------------------------------------------------
;status messages
;------------------------------------------------------------------------------
errtab1:	.db "BREAK",0,"OVERFLOW",0,"DIVIDE /0",0,"SQR FROM <0"
errtab5:	.db 0,"CONSTANT TOO BIG",0,"WRONG EXPRESSION",0,"SYNTAX ERROR",0,"UNKNOWN KEYWORD",0
errtab9:	.db "WRONG FORMAT",0,"BAD LINENUMBER",0,"NEXT W/O FOR",0,"RETURN W/O CALL"
errtab13:	.db 0,"TOO MANY CALL",0,"TOO MANY FOR",0,"I2C ERROR",0,"UNKNOWN ERROR",0
